import { MatTableDataSource, MatSort, MatTable } from '@angular/material';
import { QueryList, Injector, OnInit, OnDestroy, AfterViewInit } from '@angular/core';
import { AbstractWebWidgetComponent } from '../../abstract-web-widget.component';
import { DynamicTableInterface } from '../interfaces/dynamic-table.interface';
import { Subject } from 'rxjs';
import { DynamicTableEventDataInterface } from '../interfaces/dynamic-tale-event-data.interface';
import { DynamicTableColumnDefinitionConstants } from '../shared/dynamic-table-column-definition.constants';
import { DynamicTableActionsContainerDirective } from '../directives/dynamic-table-actions-container.directive';
import { DynamicTableIconContainerDirective } from '../directives/dynamic-table-icon-container.directive';
import { DynamicTableContentSummaryContainerDirective } from '../directives/dynamic-table-content-summary-container.directive';
import { PalTableAdditionalInfoContainerDirective } from '../directives/dynamic-table-additional-info-container.directive';
import { DynamicTableDescriptionContainerDirective } from '../directives/dynamic-table-description-container.directive';
import { DynamicTableActionMessageService } from '../services/dynamic-table-action-message.service';
import { DynamicTableDataProvider } from '../shared/dynamic-table-data-provider';
import { DynamicTableGroupExpressionInterface } from '../interfaces/dynamic-table-group-expression.interface';
import { SafeStyle, DomSanitizer } from '@angular/platform-browser';
import { DynamicTableTitleContainerDirective } from '../directives/dynamic-table-title-container.directive';
import { DynamicTableColumnInterace } from '../interfaces/dynamic-table-column.interface';
import { DynamicTableMouseEventMessageService } from '../services/dynamic-table-mouse-event-message.service';
export declare class TableInstanceComponent extends AbstractWebWidgetComponent implements OnInit, OnDestroy, AfterViewInit, DynamicTableInterface<Object> {
    static readonly FILTER_MIN_VALUE: number;
    filterHint: string;
    displayedColumns: string[];
    columns: Array<DynamicTableColumnInterace>;
    isLoadingResults: boolean;
    noDataFound: boolean;
    showFilter: boolean;
    resultsLength: number;
    dataSource: MatTableDataSource<any>;
    COLUMN_DEFINITIONS: DynamicTableColumnDefinitionConstants;
    private palTableActionsContainerDirectives;
    private palTableIconContainerDirectives;
    private palTableContentSummaryContainerDirectives;
    private palTableAdditionalInfoContainerDirectives;
    private palTableDescriptionContainerDirectives;
    private palTableTitleContainerDirectives;
    private subscription;
    private currentSelectedRowIndex;
    showSpeedDialLabels: boolean;
    palTableDataProvider: DynamicTableDataProvider;
    private palTableDynamicCellBuilder;
    private actionMessage;
    private mouseEventMessage;
    private zone;
    private changeDetectorRef;
    protected sanitizer: DomSanitizer;
    palTableIconContainerQueryList: QueryList<DynamicTableIconContainerDirective>;
    palTableActionsContainerQueryList: QueryList<DynamicTableActionsContainerDirective>;
    palTableContentSummaryContainerQueryList: QueryList<DynamicTableContentSummaryContainerDirective>;
    palTableAdditionalInfoContainerQueryList: QueryList<PalTableAdditionalInfoContainerDirective>;
    palTableDescriptionContainerQueryList: QueryList<DynamicTableDescriptionContainerDirective>;
    palTableTitleContainerQueryList: QueryList<DynamicTableTitleContainerDirective>;
    sort: MatSort;
    table: MatTable<any>;
    groupName: string;
    itemRowColor: string;
    hideColumns: boolean;
    groupExpressions: DynamicTableGroupExpressionInterface[];
    columnOptions: DynamicTableColumnInterace[];
    actionsSubject: Subject<DynamicTableActionMessageService<object>>;
    mouseEventsSubject: Subject<DynamicTableMouseEventMessageService<object>>;
    constructor(injector: Injector);
    ngOnInit(): void;
    ngAfterViewInit(): void;
    ngOnDestroy(): void;
    private load();
    addStandardColumns(): void;
    private loadData();
    private buildGroupExpressions();
    private buildGroupExpression(groupExpression);
    getItemRowColor(): SafeStyle;
    getColumnColor(columnId: string): SafeStyle;
    private hasGroups();
    private replaceItem(item);
    private removeItem(item);
    handleDataProviderEvents(data: DynamicTableEventDataInterface): void;
    isColumnDefined(column: string): boolean;
    getColumnsStyleUrl(): string;
    getIndicatorColor(item: object, rowIndex: number, groupIndex: number): string;
    getIndicatorSign(item: object, rowIndex: number, groupIndex: number): string;
    getHTMLContentSummary(item: object, rowIndex: number, groupIndex: number): string;
    getAdditionalInfo(item: object, rowIndex: number, groupIndex: number): string;
    getHTMLIcon(item: object, rowIndex: number, groupIndex: number): string;
    getTitle(item: object, rowIndex: number, groupIndex: number): string;
    getOtherTextBased(item: object, rowIndex: number, groupIndex: number): string;
    getDescription(item: object, rowIndex: number, groupIndex: number): string;
    getHTMLAction(item: object, rowIndex: number, groupIndex: number): string;
    getColumnName(columnId: string): string;
    applyFilter(filterValue: string): void;
    selectItem(index: number): void;
    getCurrentSelectedItem(): object;
    getPreviousItem(): object;
    getNextItem(): object;
    onFilterEnter(): void;
    onFilterLeave(): void;
}
