/*******************************************************************
 * @project Project: PalliaCare Mobile App                         *
 * @copyright Copyright (c) Arpage AG - 2017 - All rights reserved *
 * @address Address: Tobelweg 4, CH-8700 Küsnacht ZH               *
 * @web Web: www.arpage.ch                                         *
 * @file File: AbstractDynamicBaseService                          *
 * @author Gökhan Sari - <goekhan.sari@arpage.ch>                  *
 *******************************************************************/
import { Compiler } from "@angular/core";
/**
 * Base class for all services that want to create components dynamically.
 */
var AbstractDynamicBaseService = /** @class */ (function () {
    function AbstractDynamicBaseService(injector) {
        this.compiler = injector.get(Compiler);
    }
    /**
     * Creates a dynamic component on the fly by the given parameters.
     *
     * @param {ViewContainerRef} container
     * @param moduleToCreate
     * @param componentToCreate
     * @param properties
     * @returns {ComponentRef<any>}
     */
    AbstractDynamicBaseService.prototype.createDynamicComponent = function (container, moduleToCreate, componentToCreate, properties) {
        if (properties === void 0) { properties = {}; }
        // Compile module and components
        var mod = this.compiler.compileModuleAndAllComponentsSync(moduleToCreate);
        var factory = mod.componentFactories.find(function (comp) {
            return comp.componentType === componentToCreate;
        });
        // Create component
        var component = container.createComponent(factory);
        Object.assign(component.instance, properties);
        // If properties are changed at a later stage, the change detection
        // may need to be triggered manually:
        component.changeDetectorRef.detectChanges();
        return component;
    };
    return AbstractDynamicBaseService;
}());
export { AbstractDynamicBaseService };
//# sourceMappingURL=abstract-dynamic-base.service.js.map