var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { MatTableDataSource, MatSort, MatTable } from '@angular/material';
import { Component, ViewChild, ViewChildren, QueryList, NgZone, ChangeDetectorRef, Injector, Input } from '@angular/core';
import { AbstractWebWidgetComponent } from '../../abstract-web-widget.component';
import { DynamicTableUtils } from '../shared/dynamic-table.utils';
import { merge, of, Subject } from 'rxjs';
import { startWith, switchMap, map, catchError } from 'rxjs/operators';
import { isEqual } from 'lodash';
import { DynamicTableColumnDefinitionConstants } from '../shared/dynamic-table-column-definition.constants';
import { DynamicTableActionsContainerDirective } from '../directives/dynamic-table-actions-container.directive';
import { DynamicTableIconContainerDirective } from '../directives/dynamic-table-icon-container.directive';
import { DynamicTableContentSummaryContainerDirective } from '../directives/dynamic-table-content-summary-container.directive';
import { PalTableAdditionalInfoContainerDirective } from '../directives/dynamic-table-additional-info-container.directive';
import { DynamicTableDescriptionContainerDirective } from '../directives/dynamic-table-description-container.directive';
import { DynamicTableActionMessageService } from '../services/dynamic-table-action-message.service';
import { DynamicTableDataProvider } from '../shared/dynamic-table-data-provider';
import { DomSanitizer } from '@angular/platform-browser';
import { DynamicTableTitleContainerDirective } from '../directives/dynamic-table-title-container.directive';
import { DynamicTableDynamicCellBuilder } from '../shared/dynamic-table-dynamic-cell-builder';
import { DynamicTableMouseEventMessageService } from '../services/dynamic-table-mouse-event-message.service';
var TableInstanceComponent = /** @class */ (function (_super) {
    __extends(TableInstanceComponent, _super);
    function TableInstanceComponent(injector) {
        var _this = _super.call(this) || this;
        _this.filterHint = 'Filter';
        _this.isLoadingResults = true;
        _this.noDataFound = false;
        _this.showFilter = true;
        _this.resultsLength = 0;
        _this.dataSource = new MatTableDataSource();
        _this.COLUMN_DEFINITIONS = new DynamicTableColumnDefinitionConstants();
        _this.showSpeedDialLabels = true;
        _this.hideColumns = true;
        _this.palTableDataProvider = injector.get(DynamicTableDataProvider);
        _this.palTableDynamicCellBuilder = injector.get(DynamicTableDynamicCellBuilder);
        _this.actionMessage = injector.get(DynamicTableActionMessageService);
        _this.mouseEventMessage = injector.get(DynamicTableMouseEventMessageService);
        _this.zone = injector.get(NgZone);
        _this.changeDetectorRef = injector.get(ChangeDetectorRef);
        _this.sanitizer = injector.get(DomSanitizer);
        _this.columns = _this.palTableDataProvider.getColumns();
        _this.addStandardColumns();
        return _this;
    }
    TableInstanceComponent_1 = TableInstanceComponent;
    TableInstanceComponent.prototype.ngOnInit = function () {
        var _this = this;
        // Columns
        this.displayedColumns = DynamicTableUtils.convertToColumnIds(this.columns);
        // Subscribe to pal table events
        var eventPublisher = this.palTableDataProvider.getEventPublisher();
        if (eventPublisher != null) {
            this.subscription = eventPublisher.subscribe(function (data) { return _this.handleDataProviderEvents(data); }, function (error) { return console.error(error); });
        }
        //  Trigger data loading
        this.load();
        //
        this.actionMessage.setPublisher(this.actionsSubject.asObservable());
        this.mouseEventMessage.setPublisher(this.mouseEventsSubject.asObservable());
    };
    TableInstanceComponent.prototype.ngAfterViewInit = function () {
        // this.dataSource.sort = this.sort;
        this.load(); //  Trigger data loading
    };
    TableInstanceComponent.prototype.ngOnDestroy = function () {
        this.changeDetectorRef.detach();
        if (this.subscription != null) {
            this.subscription.unsubscribe();
        }
    };
    TableInstanceComponent.prototype.load = function () {
        var _this = this;
        setTimeout(function () {
            _this.zone.run(function () { return _this.loadData(); });
        }, 300);
    };
    TableInstanceComponent.prototype.addStandardColumns = function () {
        this.columns.unshift({ columns: ['selectedRowColumn'], names: ['selectedRowColumn'], backgroundColor: null });
    };
    TableInstanceComponent.prototype.loadData = function () {
        // If the user changes the sort order, reset back to the first page.
        // this.sort.sortChange.subscribe(() => this.paginator.pageIndex = 0);
        var _this = this;
        merge()
            .pipe(startWith({}), switchMap(function () {
            _this.isLoadingResults = true;
            return _this.palTableDataProvider.getData(_this.sort.active, _this.sort.direction, /*this.paginator.pageIndex*/ null);
        }), map(function (items) {
            // Flip flag to show that loading has finished.
            _this.isLoadingResults = false;
            if (items.length === 0) {
                _this.noDataFound = true;
            }
            else {
                _this.noDataFound = false;
            }
            _this.resultsLength = items['length'];
            return items;
        }), catchError(function (error) {
            console.error('Error in loading pal table data', error);
            _this.isLoadingResults = false;
            return of([]);
        })).subscribe(function (items) {
            if (_this.hasGroups()) {
                _this.dataSource.data = items.filter(function (item) {
                    var expression = _this.buildGroupExpressions();
                    return expression(item);
                });
                _this.isLoadingResults = false;
            }
            else {
                _this.dataSource.data = items;
                _this.isLoadingResults = false;
            }
            _this.changeDetectorRef.detectChanges();
        });
    };
    TableInstanceComponent.prototype.buildGroupExpressions = function () {
        var queryString = 'return ';
        for (var i = 0; i < this.groupExpressions.length; i++) {
            var groupExpression = this.groupExpressions[i];
            queryString += this.buildGroupExpression(groupExpression);
            if (this.groupExpressions.length > 0 && ((i + 1) !== this.groupExpressions.length)) {
                queryString += ' ' + groupExpression.operator + ' '; // combine group expressions
            }
        }
        return new Function('item', queryString);
    };
    TableInstanceComponent.prototype.buildGroupExpression = function (groupExpression) {
        var queryString = '';
        var group = groupExpression.group;
        var rules = group.rules;
        queryString += '(';
        for (var _i = 0, rules_1 = rules; _i < rules_1.length; _i++) {
            var rule = rules_1[_i];
            var operator = rule['operator'];
            var value = rule['value'];
            switch (operator) {
                case '&&':
                    queryString += ' && ';
                    break;
                case '||':
                    queryString += ' || ';
                    break;
                case '!=':
                    queryString += 'item["' + group.groupByField + '"]' + ' != ' + value;
                    break;
                case '<':
                    queryString += 'item["' + group.groupByField + '"]' + ' < ' + value;
                    break;
                case '<=':
                    queryString += 'item["' + group.groupByField + '"]' + ' <= ' + value;
                    break;
                case '>':
                    queryString += 'item["' + group.groupByField + '"]' + ' > ' + value;
                    break;
                case '>=':
                    queryString += 'item["' + group.groupByField + '"]' + ' >= ' + value;
                    break;
                case '==':
                    queryString += 'item["' + group.groupByField + '"]' + ' == ' + value;
                    break;
            }
        }
        queryString += ')';
        return queryString;
    };
    TableInstanceComponent.prototype.getItemRowColor = function () {
        return this.sanitizer.bypassSecurityTrustStyle(this.itemRowColor);
    };
    TableInstanceComponent.prototype.getColumnColor = function (columnId) {
        var column = DynamicTableUtils.getColumnById(columnId, this.columns);
        var columnOption = DynamicTableUtils.getColumnById(columnId, this.columnOptions);
        var columnColor = column['backgroundColor'];
        if (columnOption != null) {
            columnColor = columnOption['backgroundColor'];
        }
        return this.sanitizer.bypassSecurityTrustStyle(columnColor);
    };
    TableInstanceComponent.prototype.hasGroups = function () {
        return (this.groupExpressions != null && this.groupExpressions.length > 0);
    };
    /**
     * Swaps the given item after an ID comparison on the same index.
     *
     * @param item
     */
    TableInstanceComponent.prototype.replaceItem = function (item) {
        if (item !== null) {
            var filteredItems = this.dataSource.data.filter(function (itemFromList) { return isEqual(itemFromList['id'], item['id']); });
            filteredItems = (filteredItems.length > 0) ? filteredItems : this.dataSource.data.filter(function (itemFromList) { return isEqual(itemFromList['id'].elementId, item['id'].elementId); });
            if (filteredItems != null && filteredItems.length > 0) {
                // replace item in the list on the same index
                var index = this.dataSource.data.indexOf(this.dataSource.data[0]);
                this.dataSource.data[index] = item;
            }
        }
    };
    /**
     * Removes the given item from the array list.
     */
    TableInstanceComponent.prototype.removeItem = function (item) {
        if (item !== null) {
            var index = this.dataSource.data.indexOf(item);
            this.dataSource.data.splice(index, 1);
        }
    };
    /**
     * Handles events triggered from the data provider.
     *
     * @param data
     */
    TableInstanceComponent.prototype.handleDataProviderEvents = function (data) {
        switch (data['event']) {
            case 'refreshList':
                this.load();
                break;
            case 'updateItem':
                this.replaceItem(data['item']);
                break;
            case 'removeItem':
                this.removeItem(data['item']);
                break;
            case 'nextItem':
                this.actionMessage.setItem(this.getNextItem());
                this.actionMessage.setAction('nextItem');
                this.actionsSubject.next(this.actionMessage);
                break;
            case 'previousItem':
                this.actionMessage.setItem(this.getPreviousItem());
                this.actionMessage.setAction('previousItem');
                this.actionsSubject.next(this.actionMessage);
                break;
        }
    };
    TableInstanceComponent.prototype.isColumnDefined = function (column) {
        return this.displayedColumns.indexOf(column) > -1;
    };
    TableInstanceComponent.prototype.getColumnsStyleUrl = function () {
        return this.palTableDataProvider.getColumnsStyleUrl();
    };
    TableInstanceComponent.prototype.getIndicatorColor = function (item, rowIndex, groupIndex) {
        return this.palTableDataProvider.getIndicatorColor(item, rowIndex, groupIndex, this.groupName);
    };
    TableInstanceComponent.prototype.getIndicatorSign = function (item, rowIndex, groupIndex) {
        return this.palTableDataProvider.getIndicatorSign(item, rowIndex, groupIndex, this.groupName);
    };
    TableInstanceComponent.prototype.getHTMLContentSummary = function (item, rowIndex, groupIndex) {
        var _this = this;
        this.palTableContentSummaryContainerDirectives = this.palTableContentSummaryContainerQueryList.toArray();
        var htmlContentSummary = this.palTableDataProvider.getHTMLContentSummary(item, rowIndex, groupIndex, this.groupName);
        var palTableContentSummaryContainerDirective = this.palTableContentSummaryContainerDirectives[rowIndex];
        if (palTableContentSummaryContainerDirective != null) {
            this.zone.run(function () { return _this.palTableDynamicCellBuilder.createColumn(palTableContentSummaryContainerDirective.viewContainerRef, htmlContentSummary, _this.getColumnsStyleUrl(), {
                item: item,
                actionsSubject: _this.actionsSubject,
                actionMessage: _this.actionMessage,
                mouseEventsSubject: _this.mouseEventsSubject,
                mouseEventMessage: _this.mouseEventMessage
            }); });
        }
        return '';
    };
    TableInstanceComponent.prototype.getAdditionalInfo = function (item, rowIndex, groupIndex) {
        var _this = this;
        this.palTableAdditionalInfoContainerDirectives = this.palTableAdditionalInfoContainerQueryList.toArray();
        var htmlAdditionalInfo = this.palTableDataProvider.getAdditionalInfo(item, rowIndex, groupIndex, this.groupName);
        var palTableAdditionalInfoContainerDirective = this.palTableAdditionalInfoContainerDirectives[rowIndex];
        if (palTableAdditionalInfoContainerDirective != null) {
            this.zone.run(function () { return _this.palTableDynamicCellBuilder.createColumn(palTableAdditionalInfoContainerDirective.viewContainerRef, htmlAdditionalInfo, _this.getColumnsStyleUrl(), {
                item: item,
                actionsSubject: _this.actionsSubject,
                actionMessage: _this.actionMessage,
                mouseEventsSubject: _this.mouseEventsSubject,
                mouseEventMessage: _this.mouseEventMessage
            }); });
        }
        return '';
    };
    TableInstanceComponent.prototype.getHTMLIcon = function (item, rowIndex, groupIndex) {
        var _this = this;
        this.palTableIconContainerDirectives = this.palTableIconContainerQueryList.toArray();
        var iconHtml = this.palTableDataProvider.getHTMLIcon(item, rowIndex, groupIndex, this.groupName);
        var palTableIconContainerDirective = this.palTableIconContainerDirectives[rowIndex];
        if (palTableIconContainerDirective != null) {
            this.zone.run(function () { return _this.palTableDynamicCellBuilder.createColumn(palTableIconContainerDirective.viewContainerRef, iconHtml, _this.getColumnsStyleUrl(), {
                item: item,
                actionsSubject: _this.actionsSubject,
                actionMessage: _this.actionMessage,
                mouseEventsSubject: _this.mouseEventsSubject,
                mouseEventMessage: _this.mouseEventMessage
            }); });
        }
        return '';
    };
    TableInstanceComponent.prototype.getTitle = function (item, rowIndex, groupIndex) {
        var _this = this;
        this.palTableTitleContainerDirectives = this.palTableTitleContainerQueryList.toArray();
        var htmlTitle = this.palTableDataProvider.getTitle(item, rowIndex, groupIndex, this.groupName);
        var palTableTitleContainerDirective = this.palTableTitleContainerDirectives[rowIndex];
        if (palTableTitleContainerDirective != null) {
            this.zone.run(function () { return _this.palTableDynamicCellBuilder.createColumn(palTableTitleContainerDirective.viewContainerRef, htmlTitle, _this.getColumnsStyleUrl(), {
                item: item,
                actionsSubject: _this.actionsSubject,
                actionMessage: _this.actionMessage,
                mouseEventsSubject: _this.mouseEventsSubject,
                mouseEventMessage: _this.mouseEventMessage
            }); });
        }
        return '';
    };
    TableInstanceComponent.prototype.getOtherTextBased = function (item, rowIndex, groupIndex) {
        return this.palTableDataProvider.getOtherTextBased(item, rowIndex, groupIndex, this.groupName);
    };
    TableInstanceComponent.prototype.getDescription = function (item, rowIndex, groupIndex) {
        var _this = this;
        this.palTableDescriptionContainerDirectives = this.palTableDescriptionContainerQueryList.toArray();
        var htmlDescription = this.palTableDataProvider.getDescription(item, rowIndex, groupIndex, this.groupName);
        var palTableDescriptionContainerDirective = this.palTableDescriptionContainerDirectives[rowIndex];
        if (palTableDescriptionContainerDirective != null) {
            this.zone.run(function () { return _this.palTableDynamicCellBuilder.createColumn(palTableDescriptionContainerDirective.viewContainerRef, htmlDescription, _this.getColumnsStyleUrl(), {
                item: item,
                actionsSubject: _this.actionsSubject,
                actionMessage: _this.actionMessage,
                mouseEventsSubject: _this.mouseEventsSubject,
                mouseEventMessage: _this.mouseEventMessage
            }); });
        }
        return '';
    };
    TableInstanceComponent.prototype.getHTMLAction = function (item, rowIndex, groupIndex) {
        var _this = this;
        this.palTableActionsContainerDirectives = this.palTableActionsContainerQueryList.toArray();
        var actionsHtml = this.palTableDataProvider.getHTMLAction(item, rowIndex, groupIndex, this.groupName);
        var palTableActionsContainerDirective = this.palTableActionsContainerDirectives[rowIndex];
        if (palTableActionsContainerDirective != null) {
            this.zone.run(function () { return _this.palTableDynamicCellBuilder.createColumn(palTableActionsContainerDirective.viewContainerRef, actionsHtml, _this.getColumnsStyleUrl(), {
                item: item,
                actionsSubject: _this.actionsSubject,
                actionMessage: _this.actionMessage,
                mouseEventsSubject: _this.mouseEventsSubject,
                mouseEventMessage: _this.mouseEventMessage
            }); });
        }
        return '';
    };
    TableInstanceComponent.prototype.getColumnName = function (columnId) {
        return DynamicTableUtils.getColumnNameById(columnId, this.columns);
    };
    TableInstanceComponent.prototype.applyFilter = function (filterValue) {
        if (filterValue.length >= TableInstanceComponent_1.FILTER_MIN_VALUE && filterValue !== this.filterHint) {
            filterValue = filterValue.trim(); // Remove whitespace
            filterValue = filterValue.toLowerCase(); // Datasource defaults to lowercase matches
        }
        else {
            filterValue = '';
        }
        this.dataSource.filter = filterValue;
    };
    TableInstanceComponent.prototype.selectItem = function (index) {
        this.currentSelectedRowIndex = index;
        this.changeDetectorRef.detectChanges();
    };
    TableInstanceComponent.prototype.getCurrentSelectedItem = function () {
        return this.dataSource.data[this.currentSelectedRowIndex];
    };
    TableInstanceComponent.prototype.getPreviousItem = function () {
        var previousItemPosition = (this.currentSelectedRowIndex > 0) ? this.currentSelectedRowIndex - 1 : 0;
        this.selectItem(previousItemPosition);
        return this.dataSource.data[previousItemPosition];
    };
    TableInstanceComponent.prototype.getNextItem = function () {
        var nextElementPosition = (this.currentSelectedRowIndex < (this.dataSource.data.length - 1)) ? this.currentSelectedRowIndex + 1 : this.currentSelectedRowIndex;
        this.selectItem(nextElementPosition);
        return this.dataSource.data[nextElementPosition];
    };
    TableInstanceComponent.prototype.onFilterEnter = function () {
        this.filterHint = '';
    };
    TableInstanceComponent.prototype.onFilterLeave = function () {
        this.filterHint = 'Filter';
    };
    TableInstanceComponent.FILTER_MIN_VALUE = 3;
    __decorate([
        ViewChildren(DynamicTableIconContainerDirective),
        __metadata("design:type", QueryList)
    ], TableInstanceComponent.prototype, "palTableIconContainerQueryList", void 0);
    __decorate([
        ViewChildren(DynamicTableActionsContainerDirective),
        __metadata("design:type", QueryList)
    ], TableInstanceComponent.prototype, "palTableActionsContainerQueryList", void 0);
    __decorate([
        ViewChildren(DynamicTableContentSummaryContainerDirective),
        __metadata("design:type", QueryList)
    ], TableInstanceComponent.prototype, "palTableContentSummaryContainerQueryList", void 0);
    __decorate([
        ViewChildren(PalTableAdditionalInfoContainerDirective),
        __metadata("design:type", QueryList)
    ], TableInstanceComponent.prototype, "palTableAdditionalInfoContainerQueryList", void 0);
    __decorate([
        ViewChildren(DynamicTableDescriptionContainerDirective),
        __metadata("design:type", QueryList)
    ], TableInstanceComponent.prototype, "palTableDescriptionContainerQueryList", void 0);
    __decorate([
        ViewChildren(DynamicTableTitleContainerDirective),
        __metadata("design:type", QueryList)
    ], TableInstanceComponent.prototype, "palTableTitleContainerQueryList", void 0);
    __decorate([
        ViewChild(MatSort),
        __metadata("design:type", MatSort)
    ], TableInstanceComponent.prototype, "sort", void 0);
    __decorate([
        ViewChild(MatTable),
        __metadata("design:type", MatTable)
    ], TableInstanceComponent.prototype, "table", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TableInstanceComponent.prototype, "groupName", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TableInstanceComponent.prototype, "itemRowColor", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TableInstanceComponent.prototype, "hideColumns", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Array)
    ], TableInstanceComponent.prototype, "groupExpressions", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Array)
    ], TableInstanceComponent.prototype, "columnOptions", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Subject)
    ], TableInstanceComponent.prototype, "actionsSubject", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Subject)
    ], TableInstanceComponent.prototype, "mouseEventsSubject", void 0);
    TableInstanceComponent = TableInstanceComponent_1 = __decorate([
        Component({
            selector: 'table-instance-component',
            templateUrl: './table-instance.component.html',
            styleUrls: ['./table-instance.component.scss']
        }),
        __metadata("design:paramtypes", [Injector])
    ], TableInstanceComponent);
    return TableInstanceComponent;
    var TableInstanceComponent_1;
}(AbstractWebWidgetComponent));
export { TableInstanceComponent };
//# sourceMappingURL=table-instance.component.js.map